# -*- encoding: utf-8 -*-
import json
import urllib2

from rigcollector.collector import Collector

class SlushsPoolCollector(Collector):
    """
    Basic collector to call Slush's Mining Pool API
    https://mining.bitcoin.cz 
        -> My account 
        -> Manage API tokens 
        -> token
    """
    def __init__(self, conf):
        Collector.__init__(self, conf)
        self.api_url = "https://mining.bitcoin.cz/accounts/profile/json/{}".format(self.conf.get("token"))
        self.data = json.loads(urllib2.urlopen(self.api_url).read())
        
    def get_hashrate(self):
        return dict(name="Hashrate", data=dict(hashrate=float(self.data["hashrate"])))

    def get_reward(self):
        return dict(name="Reward", data=dict(reward=float(self.data["confirmed_reward"])))

    def get_all(self):
        return dict(custom=[self.get_hashrate(), self.get_reward()])



# -*- encoding: utf-8 -*-
from rigcollector.collector import Collector
import re
import logging
from datetime import datetime, timedelta

logger = logging.getLogger(__name__)

HEADER_REGEXES = ["temp", "load"]

class GpuzCollector(Collector):
    """
    Retrieve data from GPU-Z (http://www.techpowerup.com/gpuz/)
    log file.
    """

    def __init__(self, conf):
        Collector.__init__(self, conf)
        self.header_regexes = self.conf.get("header_regexes", HEADER_REGEXES)
        logging.info("GPU-Z Header regexes: " + repr(self.header_regexes))
        self.log_file = self.conf.get("log_file")
        if not self.log_file:
            logger.error("No CPU-Z log file defined")
            raise Exception("No CPU-Z log file defined")
        else:
            logger.info("CPU-Z log file: " + self.log_file)

    def parse_gpuz_log(self):
        """
        Get the first line (for header) and
        the last line (for latest data) and return
        a dict based on filter regexes.
        It also check that the latest data is not too hold.
        """
        first_line = None
        for line in open(self.log_file):
            if first_line is None:
                first_line = line
            if line != "\n":
                last_line = line
        
        headers = [h.strip() for h in first_line.split(",") if h.strip()]
        latest_data = [d.strip() for d in last_line.split(",") if d.strip()]

        latest_data_dt = datetime.strptime(latest_data[0], "%Y-%m-%d %H:%M:%S")
        if datetime.now() - latest_data_dt > timedelta(minutes=10):
            raise Exception("Latest data from GPU-Z log file is too old: " + str(latest_data_dt))

        data = {}

        combined = "(" + "|".join(self.header_regexes) + ")"
            
        for index, header in enumerate(headers):
            if re.findall(combined, header, re.IGNORECASE):
                res = latest_data[index]
                data[header.decode('mbcs', 'ignore').replace(".", "")] = float(res) if "." in res else int(res)
        
        if not data:
            logger.warning("No data found in CPU-Z log file")

        return dict(name="GPU-Z", data=data)

    def get_all(self):
        return dict(custom=[self.parse_gpuz_log()])


import psutil
import os
import time
import yaml
import codecs
import urllib2
import urllib
import json
import re
import logging
import platform
import importlib

logger = logging.getLogger(__name__)

INTERVAL = 1

def get_conf(config_file):
    if os.path.isfile(config_file):
        with codecs.open(config_file, "r", "utf8") as f:
            return yaml.load(f)
    logger.info("no config file found")
    return {}


def load_class(full_class_string):
    """
    dynamically load a class from a string
    
    >>> klass = load_class("module.submodule.ClassName")
    >>> klass2 = load_class("myfile.Class2")

    """
    class_data = full_class_string.split(".")
    module_path = ".".join(class_data[:-1])
    class_str = class_data[-1]

    module = importlib.import_module(module_path)
    # Finally, we retrieve the Class
    return getattr(module, class_str)


class Collector:
    """
    Base Collector Class, handle collector configuration
    """

    def __init__(self, conf):
        self.raw_conf = conf
        self.conf = conf.get(self.__class__.__name__, {})
        self.is_active = True

    def get_data(self):
        if self.conf.get("is_active", self.is_active):
            return self.get_all()
        logger.info(self.__class__.__name__ + "is disabled")
        return {}


class SystemCollector(Collector):
    """
    Get basic system metrics
     - uptime
     - cpu
     - mem
     - load avg
    """

    def get_utptime(self):
        with open('/proc/uptime', 'r') as f:
            return float(f.readline().split()[0])    

    def get_cpu(self):
        return psutil.cpu_percent(INTERVAL)

    def get_mem(self):
        data = psutil.virtmem_usage()
        return dict(percent=data.percent, used=int(data.used), total=int(data.total))

    def get_load(self):
        return list(os.getloadavg())

    def get_all(self):
        out = dict(system=dict(cpu=self.get_cpu(),
                                mem=self.get_mem()))

        if platform.system() != "Windows":
            out["system"]["load"] = self.get_load()
            out["system"]["uptime"] = self.get_utptime()
        
        return out


class DisksCollector(Collector):
    """
    Get disks usage informations
    """

    def get_all(self):
        res = []
        for p in psutil.disk_partitions():
            if not "opts" in p.__dict__ or p.opts != "cdrom":
                data = psutil.disk_usage(p.mountpoint)
                res.append(dict(name=p.mountpoint, 
                                data=dict(total=data.total, 
                                        used=data.used, 
                                        percent=data.percent)))
        return dict(disks=res)


class NetworkCollector(Collector):
    """
    Get upload/download for each network interface
    """

    def get_network_usage(self):
        """
        Return bytes_sent, bytes_recv for each interfaces
        """
        network_usage = {}
        for key, val in psutil.network_io_counters(pernic=True).items():
            if not re.findall(r"\.{.+}|loopback|\biface\b|\blo\b", key, re.IGNORECASE):
                network_usage[key] = dict(bs=val.bytes_sent, br=val.bytes_recv)
        return network_usage

    def get_all(self):
        """
        Return upload/download for 1 secondes in bytes
        for each interfaces
        """
        nu1 = self.get_network_usage()
        time.sleep(INTERVAL)
        nu2 = self.get_network_usage()

        res = []
        for iface in nu1.keys():
            upload = int(nu2[iface]["bs"] - nu1[iface]["bs"])
            download = int(nu2[iface]["br"] - nu1[iface]["br"])
            data = dict(upload=upload,
                        download=download)
            res.append(dict(name=iface, data=data))
        return dict(network=res)


class ProcessesCollector(Collector):
    """
    Get CPU/MEM usage for defined processes
    """

    def __init__(self, conf):
        Collector.__init__(self, conf)
        self.processes = self.conf.get("processes", [])

    def get_pid_by_name(self, name):
        for proc in psutil.process_iter():
            if name in (" ".join(proc.cmdline) or proc.name):
                return proc.pid
        return None 

    def get_process_infos(self, pid):
        p = psutil.Process(int(pid))
        return dict(mem="%.2f" % p.get_memory_percent(), 
                    cpu=p.get_cpu_percent(interval=INTERVAL))

    def get_all(self):
        res = []
        for process in self.processes:
            pid = self.get_pid_by_name(process)
            if pid:
                res.append(dict(name=process,
                                data=self.get_process_infos(pid)))
            else:
                logger.warning(process + " is not found")
        return dict(processes=res)


class CustomCollector(Collector):
    """
    Handle custom collectors (configuration and call)
    """

    def __init__(self, conf):
        Collector.__init__(self, conf)
        self.custom = self.conf.get("custom", [])

    def get_all(self):
        """
        Fetch all collector from config and
        extend all the data.
        """
        res = dict(custom=[])

        # Load each custom collector from config file
        for custom_collector in self.custom:
            try:
                CurrentCollector = load_class(custom_collector)
                res["custom"].extend(CurrentCollector(self.raw_conf).get_data()["custom"])
            except Exception, e:
                logger.error(custom_collector + " not found !")
                logger.exception(e)    
        return res


def send_data(api_url, api_key, config_file, test_mode=False):
    """
    Retrieve data for each collector and
    send data to the API via a POST request.
    """
    conf = get_conf(config_file)

    out = {}
    for col in [ProcessesCollector, 
                SystemCollector, 
                DisksCollector, 
                NetworkCollector,
                CustomCollector]:
        try:
            out.update(col(conf).get_data())
        except Exception, e:
            logger.exception(e)
    
    data = dict(ts=time.time(),
                data=out,
                api_key=api_key)
    
    if test_mode:
        return dict(data=data)

    req = urllib2.Request(conf.get("api_url", api_url),
                        json.dumps(data),
                        {'Content-Type': 'application/json'})
        
    resp = json.loads(urllib2.urlopen(req).read())
        
    return dict(status=resp["status"],
                ttw=int(resp.get("ttw", 60 * 5)),
                data=data)



# -*- encoding: utf-8 -*-
import json
import urllib2
from urllib import urlencode
import time
from hashlib import sha512
from hmac import HMAC
import base64

from rigcollector.collector import Collector


class MtGoxCollector(Collector):
    """
    Retrieve your MtGox BTC Balance
    https://mtgox.com/ 
        -> Security Center 
        -> Advanced API Key Creation 
        -> Rights: Get info

    """
    def __init__(self, conf):
        Collector.__init__(self, conf)
        self.auth_key = self.conf.get("auth_key")
        self.auth_secret = base64.b64decode(self.conf.get("auth_secret"))
 
    def get_nonce(self):
        return int(time.time()*100000)
 
    def sign_data(self, secret, data):
        return base64.b64encode(str(HMAC(secret, data, sha512).digest()))

    def build_query(self, req={}):
        req["nonce"] = self.get_nonce()
        post_data = urlencode(req)
        headers = {}
        headers["User-Agent"] = "GoxApi"
        headers["Rest-Key"] = self.auth_key
        headers["Rest-Sign"] = self.sign_data(self.auth_secret, post_data)
        return (post_data, headers)
 
    def get_btc_balance(self, path="generic/private/info", args={}):
        data, headers = self.build_query(args)
        req = urllib2.Request("https://mtgox.com/api/1/"+path, data, headers)
        res = urllib2.urlopen(req, data)
        res = json.load(res)
        return dict(name="MtGox", data=dict(balance=float(res["return"]["Wallets"]["BTC"]["Balance"]["value"])))

    def get_all(self):
        return dict(custom=[self.get_btc_balance()])



# -*- encoding: utf-8 -*-
import re
import os
import subprocess
from string import split
import logging

logger = logging.getLogger(__name__)

ATI_CONFIG_PATH = '/usr/bin/aticonfig'

from rigcollector.collector import Collector

class AtiGpuCollector(Collector):
    """
    Makes use of aticonfig to retrieve gpus temperature
    Some posts on ths topics has been helpul if you need more
    informations: https://bitcointalk.org/index.php?topic=10062.0
    """

    def __init__(self, conf):
        Collector.__init__(self, conf)
        self.ati_config_path = self.conf.get("ati_config_path", ATI_CONFIG_PATH)

    def get_gpu_temp(self, gpuid):
        """
        Retrieve temperature for a specific GPU
        """
        os.environ['DISPLAY'] = ":0"
        try:
            line = subprocess.check_output(
                [self.ati_config_path,
                 "--adapter=" + str(gpuid),
                 "--odgt"])
            m = re.search('Temperature - (\d+\.\d+)\s*C', line)
            if m is None:
                return None

            return float(m.group(1))
        except OSError:
            return None

    def get_gpus(self):
        """
        Try to get the list of all available GPUs
        """
        os.environ['DISPLAY'] = ":0"
        try:
            out = subprocess.check_output([self.ati_config_path, "--list-adapters"])
            card = 0
            gpu = 0
            lines = split(out, "\n")
            for line in lines:
                r = re.search('^[ \*]+(\d+)\. [\d:\.]+ (.+)$', line)
                if r is not None:
                    yield dict(gpuid=int(r.group(1)), desc=r.group(2))
        except OSError, e:
            logger.error("aticonfig is not available at " + self.ati_config_path)
            logger.exception(e)

    def get_gpus_temp(self):
        """
        Return data formatted for the API
        """
        gpus = []
        for gpu in self.get_gpus():
            temp = self.get_gpu_temp(gpu["gpuid"])
            if temp is not None:
                gpus.append(dict(name=gpu["desc"], data=dict(temp=temp)))
            else:
                logger.warning("No temp for " + gpu["desc"] + "GPU")
        return gpus

    def get_all(self):
        return dict(custom=self.get_gpus_temp())



import aaargh
import logging
import os
import sys
import time

from rigcollector.collector import send_data

API_URL = "https://rigsmonitoring.com/api/v1"

logger = logging.getLogger()
logger.setLevel(logging.INFO)

app = aaargh.App(description="Gather system metrics and send it to RigsMonitoring.com API.")

@app.cmd
@app.cmd_arg('-d', '--detach', type=str, default="0", help="Detach script to background")
@app.cmd_arg('--api-key', type=str, default="", help="Rig API Key")
@app.cmd_arg('-c', '--config_file', type=str, default="rigcollector.yml", help="YAML Config file")
@app.cmd_arg('-l', '--log-file', type=str, default="", help="Enable logging to file")
def collect(detach, api_key, config_file, log_file):

    detach_mode = int(detach)
    if detach_mode:
        logging.info("Detaching script to background")
        if os.fork():
            sys.exit()
    
    lh = None
    if log_file or detach_mode:          
        lh = logging.FileHandler(log_file if log_file else "rigcollector.log")
    else:
        lh = logging.StreamHandler()
    lh.setFormatter(logging.Formatter("%(asctime)s;%(name)s;%(levelname)s;%(message)s",
                              "%Y-%m-%d %H:%M:%S"))
    logger.addHandler(lh)

    while 1:
        try:
            out = send_data(API_URL, api_key, config_file)

            logger.info("Sending data:")
            logger.info(out["data"])
            logger.info(out["status"])
            
            logger.info("Sleeping for " + str(out["ttw"]) + " seconds")
            time.sleep(out.get("ttw"))
        
        except Exception, e:
            logger.exception(e)

@app.cmd
@app.cmd_arg('-c', '--config_file', type=str, default="rigcollector.yml", help="YAML Config file")
def test(config_file):
    lh = logging.StreamHandler()
    lh.setFormatter(logging.Formatter("%(asctime)s;%(name)s;%(levelname)s;%(message)s",
                              "%Y-%m-%d %H:%M:%S"))
    logger.addHandler(lh)

    out = send_data(None, None, config_file, test_mode=True)

    logger.info("Test data:")
    logger.info(out["data"])
    

def main():
    app.run()

if __name__ == '__main__':
    main()



import os
import sys
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "rigcollector",
    version = "0.1.0",
    author = "Thomas Sileo",
    author_email = "thomas.sileo@gmail.com",
    description = "Collector for rigsmonitoring.com",
    license = "MIT",
    keywords = "monitoring metrics system",
    url = "https://github.com/RigsMonitoring/rigcollector",
    packages= ['rigcollector', 'rigcollector.custom'],
    long_description= read('README.rst'),
    install_requires=[
        "aaargh", "psutil", "pyyaml"
        ],
    entry_points={'console_scripts': ["rigcollector = rigcollector.app:main"]},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
    ],
    zip_safe=False,
)


import logging
from rigcollector.collector import Collector

logger = logging.getLogger(__name__)

class DummyCollector(Collector):
    """
    Minimal custom collector example
    """

    def __init__(self, conf):
        Collector.__init__(self, conf)
        # Do some things here
        # self.my_conf_key = self.conf.get("my_conf_key", "default_value") 
        # logger.info("my custom collector initialized")

    def get_mymetric(self):
        """
        An example method wich return a dummy metric1
        Data can contains any number of data
        Metrics values must be int, float, or long
        """
        return dict(name="mymetric", data=dict(metric1=10))

    def get_all(self):
        """
        A custom collector must have get_all method,
        and must return a dict with a unique key: custom,
        wich contains a list of metrics data
        """
        return dict(custom=[self.get_mymetric()])
